from .avtovaz import AvtoVazDetails
from .bajaj import BajajDetails
from .dafra import DafraDetails
from .nissan import NissanDetails
from .opel import OpelDetails
from .renault import RenaultDetails

__all__ = [
    'AvtoVazDetails', 'BajajDetails', 'DafraDetails', 'NissanDetails', 'OpelDetails', 'RenaultDetails'
]

